#Created by Logan Balkwill
#Created on 8-30-2020
import sys
from importlib import reload as reload

import mysql.connector

#sys.path.insert(0, '.')
import PlantNannyDB.settings as s
import PlantNannyDB.dictionaries as dictionaries


#The below import statement is used in the log_information function
import PlantNannyDB.logger as logger
reload(logger)

""" TODO
        -Logging database actions
        -Clean up module (lots of redundant functions)
"""

def log_information(severity, msg,log_local=False):
    #import supporting_modules.logger as logger
    logger.log_info(log_level=severity, message=msg, local=log_local)
    
def log_locally(i, f):
    #import supporting_modules.logger as logger
    logger.log_locally(info=i,filename=f)

#Import specified database connection properties
if s.database_connection=='Local':
    db_properties=s.database_local.copy()
elif s.database_connection=='Remote':
    db_properties=s.database_remote.copy()
else:
    raise Exception("Database connection property not valid! Please update database_connection value in settings.py")

#Establish connection to database
try:
    plant_db=mysql.connector.connect(host=db_properties['hostname'],
                                     user=db_properties['db_username'],
                                     password=db_properties['db_password'],
                                     database=db_properties['db_name'])
    
    log_information(severity='p',
                    msg=("connection to database '%s' was successfull" % db_properties['db_name']))
    
except:
    log_information(severity='p',
                    msg=("FAILED to connect to database '%s'" % db_properties['db_name'])) 


def write_to_db(table, write_info,db=plant_db, log_local=True):
    #Write data to the database table
    
    #build SQL command string
    sql=build_SQL_insert(table)
    
    #insert into database
    try:
        cursor=db.cursor()
        cursor.execute(sql, write_info)
        
        db.commit()
        
        log_information(severity='p', msg=str(cursor.rowcount) + (" record inserted to %s table" % table))
    
    except:
        #failed to write to the database; store info locally
        log_information(severity='p', msg="Failed to write to database")
        if log_local==True:
            log_locally(i=write_info, f=table)
        else: raise Exception("Information not logged!!!")

def query_db(sql_string):
    #Handles database queries; returns query results
    results=[]

    try:
        #Validate the query string
        if (sql_string.find('SELECT')==-1) and (sql_string.find('select')==-1):
            #INVALID!! the provided string may be a 'UPDATE','DELETE', or other style string
            raise Exception('Error retrieving info from database... SQL string does not contain a SELECT statement!!')

        #Run Query
        mycursor=plant_db.cursor()
        mycursor.execute(sql_string)
        
        results=mycursor.fetchall()
        mycursor.close()

        return results

    except Exception as err:
        #Could not process query; Log error info locally

        log_information(severity='p', msg="Failed to query database!!")
        log_information(severity='s', msg=err, log_local=True)

        return results


def build_SQL_insert(table_name):
    #returns sql string of command
    
    if table_name in sql_insert:
        return sql_insert[table_name]
    else:
        return ''

def get_sql_dictionary(category):
    # build dictionaries from database if possible. otherwise default to package-stored dictionary
    sql = 'SELECT id_scriptname, script FROM def_SQLscripts WHERE active=1 and script_type="' + category + '"'
    
    try:
        dic = query_db(sql)
        return dict(dic)
    except:
        print("WARNING: unable to query database while attempting to build libraries. Using default libraries...")
        if category == "INSERT":
            return dictionaries.sql_insert.copy()
        elif category == "SELECT":
            return dictionaries.sql_select.copy()
        else:
            raise Exception("Unexpected value passed in when trying to build SQL dictionaries. Expected SELECT or INSERT. Value supplied - ", category)
    
def get_sensor_list(additional_sql):
    #returns list of active sensors
    
    #build SQL query string
    sql= sql_select['sensors']
    
    if len(additional_sql)>1:
        sql=sql + " WHERE " + additional_sql 
        
    #execute query
    return query_db(sql_string=sql)

def get_sensor_frequencies(additional_sql):
    #returns list of sensor read frequencies
    
    #build SQL query string
    sql= sql_select['sensor_freq']
    
    if len(additional_sql)>1:
        sql=sql + " WHERE " + additional_sql 
        
    #execute query
    return query_db(sql_string=sql)


def build_plant_devices_list():
    #Used to build the main list of plant devices and characteristics used in program
    #Output tuple as: PlantID, Plant Name, Device_ID, Device Name, Action_Frequency_Min
    
    #Get SQL string
    sql=sql_select['device_assignments'] 

    #execute query
    return query_db(sql_string=sql)

#Import SQL dictionaries:
sql_insert=get_sql_dictionary("INSERT")
sql_select=get_sql_dictionary("SELECT")

if __name__=="__main__":
    log_information(severity='p', msg="Attempting to write to DB")
    write_to_db(db=plant_db,table='soilsensor_trans',write_info=['2020-08-31','1',20,390])



#settings.py

"""---------QUICK-EDIT SETTINGS---------"""
database_connection='Local'

#Interval for checking for new information
refresh_mins=10
#Instrument Settings
read_frequency_mins=1



"""-------ADVANCED CONFIG SETTINGS-------"""
#MySQL Database Information (Local):
database_local={
    'db_name'       :   'plantnanny',    
    'hostname'      :   'localhost',
    'db_username'   :   'mainuser',
    'db_password'   :   'mainuser'
}
phpmyadmin_password='Balk45610'

#MySQL Database Information (Remote):
database_remote={
    'db_name'       :   'plantnan_plantnanny',       #cpanelUsername_databaseName
    'hostname'      :   '50.87.253.14:3306',        #server IP Address:connection port
    'db_username'   :   'plantnan_raspberrypi',  #cpanelUsername_databaseUsername
    'db_password'   :   'FBoIxeR7bB=b'          #database User Password
}

#Local Logging Information:
#(only used to store data if MySQL database is unavailable)
log_directory='/home/pi/Documents/Plant_Nanny/Logs/'
file_suffix='.txt'

#Log levels permitted; comment out any undesired level
log_levels=(
    'i', #informational
    'w', #warning
    #'d', #debugging
    's', #severe
    #'p', #print (prints to the shell; not logged into database) DO NOT UNCOMMENT!!!
    )
    
#I2C Instruments
addr_sensor_soil=0X36
addr_sensor_gas=0X58

#Camera Information
image_directory='/home/pi/Documents/Plant_Nanny/Plant-Photos/'
image_filetype='.jpg'

#Graphics Information
graphics_directory='/home/pi/Documents/Plant_Nanny/Graphics/'
graphics_filetype='.png'




""" Database-Related Dictionaries:  """

#Writing information to database
sql_insert={
    'soilsensor_trans'  :   "INSERT INTO soilsensor_trans (DateTime, plant_id, SoilTemp_DegC, SoilMoisture_val) VALUES (%s, %s, %s, %s)",
    'gassensor_trans'   :   "INSERT INTO `gassensor_trans` (`record_id`, `DateTime`, `room_id`, `eCO2_ppm`, `TVOC_ppb`) VALUES (NULL, %s, %s, %s, %s)",
    'lightsensor_trans' :   "INSERT INTO `lightsensor_trans` (`record_id`, `DateTime`, `room_id`, `colour_red`, `colour_green`, `colour_blue`, `colour_clear`) VALUES (NULL, %s, %s, %s, %s, %s, %s)",
    'airsensor_trans'   :   "INSERT INTO `airsensor_trans` (`record_id`, `DateTime`, `room_id`, `AirTemp_DegC`, `AirHumidity_percent`, `AirGas_ohms`, `AirPressure_hpa`) VALUES (NULL, %s, %s, %s, %s, %s, %s)",
    'photo_trans'       :   "INSERT INTO `photo_trans` (`record_id`, `capture_datetime`, `plant_id`, `image_path`) VALUES (NULL, %s, %s, %s)",
    'log_trans'         :   "INSERT INTO `log_trans` (`record_id`, `datetime`, `log_type`, `log_message`) VALUES (NULL, %s, %s, %s)"
}           

#Retrieving information from database
sql_select={
    'air_temp'          :   "SELECT CAST(a.DateTime as Time) as Timestamp, a.AirTemp_DegC FROM airsensor_trans a WHERE a.DateTime >= '%s' AND a.DateTime < '%s' AND a.room_id= '%s'",
    'air_humidity'      :   "SELECT CAST(a.DateTime as Time) as Timestamp, a.AirHumidity_percent FROM airsensor_trans a WHERE a.DateTime >= '%s' AND a.DateTime < '%s' AND a.room_id= '%s'",
    'sensors'           :   "SELECT * FROM sensors",
    'sensor_freq'       :   "SELECT read_frequency_min FROM sensors",
    'device_assignments':   "SELECT DA.`id_room` AS Assigned_ID, 'Environment' AS Assigned_Type, E.`name_short` AS Assigned_Name, DA.`id_device`, D.`model` AS Device_Name, COALESCE(DA.`action_freq_mins`, D.`default_action_freq_mins`) AS Action_Frequency FROM `def_device_assignments` AS DA LEFT JOIN `def_devices` AS D ON DA.`id_device`=D.id LEFT JOIN `def_environments` AS E ON DA.id_room=E.id_room WHERE DA.`active`=1 AND E.`active`=1 AND D.`supported`=1 AND DA.id_room>0 UNION SELECT DA.`id_plant` AS Assigned_ID, 'Plant' AS Assigned_Type, P.`name` AS Assigned_Name, DA.`id_device`, D.`model` AS Device_Name, COALESCE(DA.`action_freq_mins`, D.`default_action_freq_mins`) AS Action_Frequency FROM `def_device_assignments` AS DA LEFT JOIN `def_devices` AS D ON DA.`id_device`=D.id LEFT JOIN `def_plants` AS P ON DA.id_plant=P.id WHERE DA.`active`=1 AND P.`active`=1 AND D.`supported`=1 AND DA.id_plant>0"
}

""" Error Handling Dictionaries:    """

#Error Statements
err_statement={}    #TODO: Complete Dictionary

if __name__=="__main__":
    x=sql_select['air_humidity']
    print(x)


from datetime import datetime
#import ast
import os, sys
sys.path.insert(0, '..')

import PlantNannyDB.settings as settings
import PlantNannyDB.database_use as db


def log_action(event,result, additional_info=''):
    #break apart event into components
    Assigned_id, Assigned_type, Assigned_name, device_id, device_name, action_freq=event
    
    
    #create writing information
    if result=='success':
        severity='d' #informational
        message_prefix='Successfully performed device action(s): '
    elif result=='failure':
        severity='w' #warning
        message_prefix='Failed to perform device action(s): '
        
    message_base= 'Assigned_type= %s, Assigned_id= %s, Assigned_name= %s, device_id= %s, device_name= %s' % (Assigned_type, Assigned_id, Assigned_name, device_id, device_name)
    
    message=message_prefix + message_base
    
    if additional_info!='':
        message=message + '; ' + additional_info
            
    log_info(severity,message)
    
    
def log_info(log_level,message, filename_log='log_trans', local=False):
    dt=datetime.now()

    if log_level=='p':
        print('(' + dt.strftime("%m/%d/%Y, %H:%M:%S") + '): ' + message)
    elif local==True:
        msg=[dt.strftime("%m/%d/%Y, %H:%M:%S"), log_level, message]
        log_locally(info=msg, filename=filename_log)
    elif log_level in settings.log_levels:
        db.write_to_db(table='log_trans',
                                 write_info=[datetime.now(), log_level, message])


def log_locally(info, filename, folder_path=settings.log_directory, filetype=settings.file_suffix):
    #called when database is unavailable
    
    fullpath=folder_path + filename + filetype
    
    #Make directory if doesnt already exist
    if os.path.exists(folder_path)==False:
        os.mkdir(folder_path)
    
    #Build string
    msg=''
    for item in info:
        if item==info[0]:
            msg+=str(item)
        else:
            msg+= '; ' + item
    
    #Write information to file
    f=open(fullpath,"a")
    f.write(msg + '\n')
    f.close()

def local_logs_exist():
    #checks if any queue exists for uploading data to database
    directory=settings.log_directory
    
    count=0
    logfiles=0
    otherfiles=0
    
    for filename in os.listdir(directory):
        if filename.endswith(".txt") or filename.endswith(".csv"):
            logfiles+=1
            f=open(directory+filename,"r")
            for line in f:
                count+= 1
        else:
            otherfile+=1
            
    if otherfiles!=0:
        msg="%s log(s) exist between %s log file(s) (%s non-log files exist)" % (count, logfiles, otherfiles)
    else:
        msg="%s log(s) exist between %s log file(s)" % (count, logfiles)
        
        
    return count, msg


def upload_local_logs():
#attempt to upload queued logs to database
    log_info(log_level = 'p', message = "Attempting to upload queued logs....")

    directory=settings.log_directory
    logfiles=0
    count=0
    logs_success=0
    logs_failed=0
    
    #loop through log files
    for filename in os.listdir(directory):
        if filename.endswith(".txt") or filename.endswith(".csv"):
            logfiles+=1
            table_name=os.path.splitext(filename)[0]
            
            #Below section opens file (read) and stores info in variable
            #Next, the file is opened (write) to overwrite the file
            #    ONLY info that couldn't be uploaded
            with open(directory+filename,"r") as f:
                #loop through log lines
                lines = f.readlines()
            with open(directory+filename, "w") as f:
                for line in lines:
                    count+= 1
                    
                    #extract information
                    line=str(line)
                    list_str=line.rstrip()
                    lst=list_str.split('; ')
                    
                    #attempt to upload to db
                    try:
                        db.write_to_db(table=table_name,
                                       write_info=lst,
                                       log_local=False)
                        logs_success+=1
                    except:
                        f.write(line)
                        logs_failed+=1
    
    #Build result message
    msg='Results of re-uploading queued logs: %s successful,  %s failed (%s files involved)' % (logs_success, logs_failed, logfiles)                   
    log_info(log_level='p',message=msg)  
                

if __name__=="__main__":
    local_logs_exist()


from setuptools import setup, find_packages

try:
    with open('README.md') as f:
        readme = f.read()
except:
    pass

try:
    with open('LICENSE.txt') as f:
        license = f.read()
except:
    pass

setup(
    name='PlantNannyDB',
    version='1.4.6',
    description='Enable read/write functionality for Plant Nanny Database',
    long_description="README.md",
    long_description_content_type = 'text/markdown',
    author='Logan Balkwill',
    author_email='lgb0020@gmail.com',
    url='https://github.com/loganbalkwill/plant-nanny-db',
    license="MIT",
    packages=find_packages(exclude=('tests', 'sensors')),
    include_package_data = True,
    install_requires = ["mysqlclient", "importlib"]
)